// ●リスト10::Generatorによる非同期処理
// (d1)本体のフロー
asyncflow(function *() {
  // (d2)yieldにPromiseを渡す
  var items = yield getUrl("/items");
  var id = items[0].id;
  var item = yield getUrl("/items/" + id);
  console.log(item);
});

// フロー制御関数
function asyncflow(generator) {
  var g = generator();
  var next = value => {
    // (d3)Promiseを受け取る
    var result = g.next(value);
    if (!result.done) {
      var promise = result.value;
      promise.then(value => {
        // (d4)Promiseが完了したらnext関数に結果を渡す
        next(value);
      });
    }
  };
  next();
}

function getUrl(url) {
  return new Promise((resolve, reject) => {
    var xhr = new XMLHttpRequest();
    xhr.open("GET", url);
    xhr.onload = () => {
      if (xhr.status === 200) {
        resolve(xhr.statusText);
      } else {
        // (d1)エラーの場合rejectを呼ぶ
        reject(new Error(xhr.statusText));
      }
    };
    xhr.onerror = () => {
      // (d2)エラーの場合rejectを呼ぶ
      reject(new Error(xhr.statusText));
    };
    xhr.send();
  });
}
